/*
-----------------------------------------------------------
 Copyright (c) Youssef Hmamouche - All Rights Reserved.

 This software is subject to the General Public License.
 ----------------------------------------------------------
*/

/*
 * file: commit_script.c
 * This file contains the implementation of functions used to dump a scriptable 
 * form of the commands issued by the user.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <errno.h>

#include "dbg.h"
#include "commit_script.h"

char* script_path;
int dump_success;

#ifdef DEBUG
extern FILE* f;
#endif

GList* cmd_list;

extern int lkcp_mode;

/* Append command/data into doubly-list. This function executes every time a user
 * commits a change to a field/value. This function is called in entry_lookup.c.
 */

void log_cmd(const char* path, const char* cmd){
	s_commit* commit = g_new(s_commit, 1);

	LOG_CALL(0);

	if(commit == NULL){
		LOG_CALL(1);
		return;
	}
	memset(commit->path, 0, sizeof(commit->path));
	memset(commit->cmd, 0, sizeof(commit->cmd));

	if(path == NULL || cmd == NULL){
		g_free(commit);
		LOG_CALL(1);
		return;
	}

	strncpy(commit->path, strdup(path), strlen(path));
	strncpy(commit->cmd, strdup(cmd), strlen(cmd));

	cmd_list=g_list_append(cmd_list, commit);
#ifdef DEBUG
                fprintf(f, "file: %s\n", ((s_commit*)cmd_list->data)->path);
#endif

	LOG_CALL(1);
}

/* 
 * write unique commands to script file. This function is called when the menu
 * option 'D' is pressed. First, we copy the current list of commands, delete 
 * duplicate commands, reverse the commands to select the recently issued one.
 */

void dump_script(void){	/* use g_list_prepend so that the last item stays last */
	
	GList* dump_list=NULL;
	LOG_CALL(0);
	
	if(cmd_list == NULL){
		dump_success=0;
		LOG_CALL(1);
		return;
	}
	dump_list = g_list_copy(cmd_list);	/* keep original list clean */

	dump_list=del_dup_cmd(dump_list);

	write_cmd_tofile(dump_list);

	g_list_free(dump_list);

	LOG_CALL(1);
}

GList* del_dup_cmd(GList* dump_list){
	GList* start=NULL, *end=NULL, *new_list=NULL, *temp=NULL;
	LOG_CALL(0);

	if(dump_list == NULL){
		dump_success=0;
		LOG_CALL(1);
		return NULL;
	}

	dump_list=g_list_reverse(dump_list);	/* Reverse it to start from last element, the most relevant one.
						 * The reason being that we only care about the last written dup.
						 */
#ifdef DEBUG
        start=g_list_first(dump_list);
        end=g_list_last(dump_list);
        for(temp=start; temp!=end->next; temp=temp->next)
                fprintf(f, "items before: %s\n", ((s_commit*)temp->data)->path);
#endif

	start=g_list_first(dump_list);
	end=g_list_last(dump_list);
	
	for(temp=start; temp!=end->next; temp=temp->next){
		if(temp == NULL)
			continue;
		if(temp == start){
			new_list=g_list_append(new_list, (s_commit*) temp->data);
		}
		else {
			if(g_list_find_custom(new_list, ((s_commit*)temp->data)->path, (GCompareFunc) strcmp) == NULL){
				new_list = g_list_append(new_list, ((s_commit*) temp->data));
			}
		}
	}
	dump_list=NULL;
	dump_list=g_list_reverse(new_list);	/* put the list back to its original order */

#ifdef DEBUG
	start=g_list_first(dump_list);
	end=g_list_last(dump_list);
	for(temp=start; temp!=end->next; temp=temp->next)
		fprintf(f, "item: %s\n", ((s_commit*)temp->data)->path);
#endif
	LOG_CALL(1);
	
	return dump_list;
}

	/*
	 * write_cmd_tofile dumps the commands into a bash scriptable file. The variables are the path
	 * and the command to the virtual file. The macros are there to write constant strings to the
	 * file. Ex:
	 * CREATE_OUTPUT_FILE
	 * OUTPUT_HEADER -> 		#!/bin/bash
	 * OUTPUT_CHECK_FILE ->		if [ -f
	 * etc...
	 */
void write_cmd_tofile(GList* dump_list){
	
	FILE* fhandle=NULL;
	int fd;
	char file[]="/tmp/lkcp_dump.shXXXXXX";
	GList *end=NULL, *first=NULL, *temp=NULL;

	LOG_CALL(0);

	 if(dump_list == NULL){
		dump_success=0;
                LOG_CALL(1);
                return;
        }

#ifdef DEBUG
        first=g_list_first(dump_list);
        end=g_list_last(dump_list);
        for(temp=first; temp!=end->next; temp=temp->next)
                fprintf(f, "item: %s\n", ((s_commit*)temp->data)->path);
#endif
	
	fd=mkstemp(file);
	
	if(fd < 0){
		dump_success=0;
		LOG_CALL(1);
		return;
	}
	
	if((fhandle=fdopen(fd, "a")) == NULL){
		dump_success=0;
		LOG_CALL(1);
		return;
	}
	first = g_list_first(dump_list);
	if(first == NULL){
		dump_success=0;
		fclose(fhandle);
		LOG_CALL(1);
		return;
	}
	end = g_list_last(dump_list);

	if(end == NULL){
		fclose(fhandle);
		LOG_CALL(1);
		return;
	}
	OUTPUT_HEADER(fhandle);

	for(temp=first; temp!= end->next; temp=temp->next){
#ifdef DEBUG
                fprintf(f, "file: %s - i: %d\n", ((s_commit*)temp->data)->path, g_list_position(dump_list, temp));
#endif
		OUTPUT_CHECK_FILE(fhandle);
		OUTPUT_PROC_FILENAME(fhandle,((s_commit*)temp->data)->path);
		OUTPUT_HEADER_CONTINUE(fhandle);
		OUTPUT_ECHO(fhandle);
		OUTPUT_FIELD_WRITE_VALUE(fhandle,((s_commit*)temp->data)->cmd);
		OUTPUT_PROC_FILENAME_REDIR(fhandle,((s_commit*)temp->data)->path);
		OUTPUT_FOOTER(fhandle);
	}
	dump_success=1;	
	script_path = strdup(file);
	fclose(fhandle);	

	LOG_CALL(1);
}

void draw_dumpwin(WINDOW* dumpwin){
	int maxy=0, maxx=0;
	LOG_CALL(0);
	getmaxyx(dumpwin, maxy, maxx);
	(void) box(dumpwin, ACS_VLINE, ACS_HLINE);
	(void) wbkgd(dumpwin, COLOR_PAIR(12));
	(void) wmove(dumpwin, 1, 1);
	(void) wattron(dumpwin, A_BOLD);
        if(dump_success == 1){
                (void) wprintw(dumpwin, "Script was successfully dumped into %s", script_path);
        }
        else {
		if(lkcp_mode == VIEW_MODE){
			(void) wprintw(dumpwin, "This feature is available to root only.");
		}
		else{
                	(void) wprintw(dumpwin, "Error dumping script. Have you changed any field/value yet?");
		}
        }
        (void) wattroff(dumpwin, A_BOLD);
        (void) wrefresh(dumpwin);
        sfree(script_path);
        LOG_CALL(1);
}

void draw_dumpban(WINDOW* dumpban){
        int maxy=0, maxx=0;
        LOG_CALL(0);
        getmaxyx(dumpban, maxy,  maxx);
        (void) box(dumpban, ACS_VLINE, ACS_HLINE);
        (void) wbkgd(dumpban, COLOR_PAIR(1));
        (void) wmove(dumpban, 1, 1);
        (void) wattron(dumpban, A_BOLD);
        (void) wprintw(dumpban, "Press the Escape key (Esc) to exit this screen");
        (void) wattroff(dumpban, A_BOLD);
        (void) wrefresh(dumpban);
        LOG_CALL(1);
}

